"use strict";

// PLACEHOLDER-ADEIRA-UNIVERSE-INTERNAL


var template = require('@babel/template').default;

var murmurHash = require('@adeira/murmur-hash').default;

var childProcess = require('child_process');

var findCacheDir = require('find-cache-dir');

var fs = require('fs');

var path = require('path');

var hash = require('object-hash');

var getCssDeclarations = require('./getCssDeclarations').default;

var TemplateLiteralHandler = require('./TemplateLiteralHandler').default;

var StringLiteralHandler = require('./StringLiteralHandler').default;

var ConditionalExpressionHandler = require('./ConditionalExpressionHandler').default;

var PLUGIN_NAME = 'sx-tailwind-babel-transform';

module.exports = function sxTailwindBabelPlugin() {
  var stylesCollector = [];
  var stylesVarName = 'styles';
  var styles = {};
  var keyframes = {};
  var lastImportPointer;
  var sxImported = false;
  return {
    name: PLUGIN_NAME,
    pre: function pre(state) {
      var cacheDir = findCacheDir({
        name: PLUGIN_NAME
      });

      if (!fs.existsSync(cacheDir)) {
        fs.mkdirSync(cacheDir, {
          recursive: true
        });
      }

      var options = state.opts.plugins.find(function (p) {
        return p.key === PLUGIN_NAME;
      }).options;
      var cacheFile = path.join(cacheDir, "styles_".concat(hash(options), ".json"));

      if (!fs.existsSync(cacheFile)) {
        // This way async code can be called in sync process (Babel is sync, Tailwind processor async)
        childProcess.execFileSync('node', ["".concat(getCurrentDir(), "createCache.js"), JSON.stringify({
          options: options,
          cacheFile: cacheFile
        })]);
      }

      var cache = require(cacheFile);

      keyframes = cache.keyframes;
      styles = cache.styles;
    },
    visitor: {
      JSXAttribute: function JSXAttribute(path) {
        if (path.node.name.name !== 'sxt') {
          return;
        }

        path.traverse({
          TemplateLiteral: function TemplateLiteral(path) {
            TemplateLiteralHandler(path, stylesCollector, stylesVarName);
          }
        });

        if (path.node.value.type === 'JSXExpressionContainer' && path.node.value.expression.type === 'ConditionalExpression') {
          ConditionalExpressionHandler(path, stylesCollector, stylesVarName);
        }

        if (path.node.value.type === 'StringLiteral') {
          StringLiteralHandler(path, stylesCollector, stylesVarName);
        }
      },
      ImportDeclaration: function ImportDeclaration(path) {
        if (path.node.source.value === '@adeira/sx' && path.node.specifiers.find(function (s) {
          return s.type === 'ImportDefaultSpecifier' && s.local.name === 'sx';
        })) {
          sxImported = true;
        }

        lastImportPointer = path;
      },
      Program: {
        enter: function enter(path, state) {
          sxImported = false;
          stylesCollector = [];
          var filename = state.file.opts.filename.split('/').slice(-2).join('/');
          stylesVarName = "__styles_".concat(murmurHash(filename));
        },
        exit: function exit(path) {
          var declarations = Object.fromEntries(stylesCollector.map(function (style) {
            return [style, getCssDeclarations(style, keyframes, styles)];
          }));

          if (Object.keys(declarations).length === 0) {
            return;
          }

          if (sxImported === false) {
            lastImportPointer.insertAfter(template.ast("import sx from '@adeira/sx'"));
          }

          path.node.body.push(template.ast("const ".concat(stylesVarName, " = sx.create(").concat(JSON.stringify(declarations), ")"))); // if animation is used, transform "sx.keyframe" string into call expression

          path.traverse({
            ObjectProperty: function ObjectProperty(path) {
              if (typeof path.node.key.value !== 'string' || !path.node.key.value.startsWith('--animation-name-')) {
                return;
              }

              var sxKeyframe = template.ast(path.node.value.value);
              path.node.value = sxKeyframe.expression;
            }
          });
        }
      }
    }
  };
};

function getCurrentDir() {
  var possibleDirs = ['./src/babel-plugin-transform-sx-tailwind/src/', // Adeira Universe monorepo
  './node_modules/@adeira/babel-plugin-transform-sx-tailwind/src/', // NPM package
  '../../node_modules/@adeira/babel-plugin-transform-sx-tailwind/src/', // standard monorepo
  '../babel-plugin-transform-sx-tailwind/src/' // Vercel CI
  ];
  var dir = possibleDirs.find(function (d) {
    return fs.existsSync(d);
  });

  if (dir != null) {
    return dir;
  }

  throw Error("Error, can't detect current working dir! cwd: ".concat(process.cwd()));
}